
/* --------------------------------------------- */
/*  H8-3069F File access (FAT16/32) function     */
/*                                               */
/*  CPU    : Renesus H8/3069F 25MHz              */
/*  Memory : ROM 512KB, RAM 16KB E-RAM 2MB       */
/*                (c) KAZ.Imamura                */
/* --------------------------------------------- */

#ifndef __FAT_H__
#define __FAT_H__

#include "common.h"
#include "ui.h"
#include "usb_ms.h"

#define MAX_FILE_IN_DIR		128
#define WINDOW_SIZE			512

#define DIR_ATTR_READONLY			0x01
#define DIR_ATTR_HIDDEN				0x02
#define DIR_ATTR_SYSTEM				0x04
#define DIR_ATTR_VOLUME_ID			0x08
#define DIR_ATTR_DIRECTORY			0x10
#define DIR_ATTR_ARCHIVE			0x20
#define DIR_ATTR_LONG_NAME			(DIR_ATTR_READONLY|DIR_ATTR_HIDDEN|DIR_ATTR_SYSTEM|DIR_ATTR_VOLUME_ID)

// Keep file list on current directory
typedef struct {
	unsigned long	FirstCluster;
	unsigned long	FileSize;
	char			Name[13];
	unsigned char	Attributes;
} DIR_INFO, *P_DIR_INFO;

// Read file status
typedef struct {
	unsigned long ReadSize;
	unsigned long FileSize;
	unsigned long CurrentCluster;
	unsigned long CurrentSector;
} READ_FILE_STS, *P_READ_FILE_STS;

typedef struct {
	unsigned char fat_type;
	unsigned long fat_BPB_sector;
} MBR_INFORMARION, *P_MBR_INFORMATION;

typedef struct {
	unsigned int  ResvdSecCnt;				// Offset 24 Size 2  
	unsigned int  BytsPerSec;				// Offset 11 Size 2
	unsigned char NumFATs;					// Offset 16 Size 1
	unsigned int  RootEntCnt;				// Offset 17 Size 2 (Only FAT16 has valid data, FAT32 has 0.)
	unsigned int  FATSz16;					// Offset 22 Size 2 FAT size for FAT16
	unsigned int  TotSec16;					// Offset 19 Size 2
	unsigned long TotSec32;					// Offset 32 Size 4
	unsigned char SecPerClus;				// Offset 13 Size 1
											// Only for FAT32
	unsigned long FATSz32;					// Offset 36 Size 2 FAT size for FAT32
	unsigned long RootClus;					// Offset 44 Size 4
} BPB_INFORMARION, *P_BPB_INFORMATION;


// Global variables
extern unsigned int  TotalFileNum;
extern READ_FILE_STS ReadFileStatus;
extern DIR_INFO CurrentDirInfo[MAX_FILE_IN_DIR];		// Keep file list on current directory

// Global functions
extern void fat_20ms_handler(void);
extern int  fat_initialize(void);
extern int  fat_process(void);
extern int  ui_function_fat(UI_COMMAND uicmd);
extern unsigned char fat_status(REQUEST_TO_CLASS req);
extern unsigned char fat_change_directory( unsigned long cluster );
extern unsigned char fat_read_file( unsigned long cluster, unsigned long file_size );



#endif
